@@ -101,18 +101,27 @@ def read_frequency(expr=None, genes=None, Xcen=None, frequency_file=None, read_f
 				freq.append(int(ll[1]))
 		f.close()
 
-	os.system("cd %s && Rscript do_kmeans.R gene.freq.good.txt 1 %d 1000 km_centroid.txt km_label.txt" % (outdir, num_query_sizes))
+	if len(sizes)<=num_query_sizes:
+		sys.stderr.write("Setting num_query_sizes to be %d instead of %d...\n" % (len(sizes), num_query_sizes))
+		sys.stderr.flush()
+		num_query_sizes = len(sizes) 
 
-	f = open("%s/km_centroid.txt")
+	cmd = "cd '%s' && Rscript do_kmeans.R gene.freq.good.txt 1 %d 1000 km_centroid.txt km_label.txt" % (outdir, num_query_sizes)
+	#print(cmd)
+	os.system(cmd)
+
+	#os.system("cd '%s' && Rscript do_kmeans.R gene.freq.good.txt 1 %d 1000 km_centroid.txt km_label.txt" % (outdir, num_query_sizes))
+
+	f = open("%s/km_centroid.txt" % outdir)
 	cc = []
 	for l in f:
 		l = l.rstrip("\n")
 		ll = l.split(" ")
-		cc.append(int(ll[1]))
+		cc.append(int(float(ll[1])))
 	f.close()
 
 	labels = []
-	f = open("%s/km_label.txt")
+	f = open("%s/km_label.txt" % outdir)
 	for l in f:
 		l = l.rstrip("\n")
 		ll = l.split(" ")
