@@ -4,20 +4,21 @@ import re
 import numpy as np
 import math
 import scipy
-import spatial_genes
+import silhouetteRank
+import silhouetteRank.spatial_genes as spatial_genes
 import subprocess
 from shutil import copyfile
 from operator import itemgetter
 from scipy.spatial.distance import squareform, pdist
 from scipy.stats import percentileofscore
 from sklearn.metrics import roc_auc_score
-from sklearn.cluster import KMeans
 
 def read_matrix():
 	f_expr = "expression.txt"
 	f_Xcen = "Xcen.good"
 
 	sys.stderr.write("Reading gene expression...\n")
+	sys.stderr.flush()
 	f = open(f_expr)
 	h = f.readline().rstrip("\n").split("\t")[1:]
 	num_cell = len(h)
@@ -42,6 +43,7 @@ def read_matrix():
 	f.close()
 
 	sys.stderr.write("Reading cell coordinates...\n")
+	sys.stderr.flush()
 	f = open(f_Xcen)
 	Xcen = np.empty((num_cell, 2), dtype="float32")
 	f.readline()
@@ -54,7 +56,7 @@ def read_matrix():
 	f.close()
 	return expr, genes, Xcen
  
-def read_frequency(expr=None, genes=None, Xcen=None, frequency_file=None, read_from_file=True, outdir="", examine_top=0.05):
+def read_frequency(expr=None, genes=None, Xcen=None, frequency_file=None, read_from_file=True, outdir="", examine_top=0.05, num_query_sizes=10):
 	num_cell = Xcen.shape[0]
 	ncell = num_cell
 	ex = int((1.0-examine_top)*100.0)
@@ -98,12 +100,31 @@ def read_frequency(expr=None, genes=None, Xcen=None, frequency_file=None, read_f
 				freq.append(int(ll[1]))
 		f.close()
 
-	kmeans = KMeans(n_clusters = 10, random_state=0, n_init=1000, verbose=0).fit(np.array([freq]).T)
+	if len(sizes)<=num_query_sizes:
+		sys.stderr.write("Setting num_query_sizes to be %d instead of %d...\n" % (len(sizes), num_query_sizes))
+		sys.stderr.flush()
+		num_query_sizes = len(sizes) 
+
+	os.system("cd '%s' && Rscript do_kmeans.R gene.freq.good.txt 1 %d 1000 km_centroid.txt km_label.txt" % (outdir, num_query_sizes))
+
+	f = open("%s/km_centroid.txt" % outdir)
 	cc = []
-	for c in kmeans.cluster_centers_:
-		cc.append(int(c[0]))
+	for l in f:
+		l = l.rstrip("\n")
+		ll = l.split(" ")
+		cc.append(int(float(ll[1])))
+	f.close()
+
+	labels = []
+	f = open("%s/km_label.txt" % outdir)
+	for l in f:
+		l = l.rstrip("\n")
+		ll = l.split(" ")
+		labels.append(int(ll[1]) - 1)
+	f.close()
+
 	by_cluster = {}
-	for ind,c in enumerate(kmeans.labels_):
+	for ind,c in enumerate(labels):
 		by_cluster.setdefault(cc[c], [])
 		by_cluster[cc[c]].append(freq[ind])
 
@@ -180,8 +201,11 @@ if __name__=="__main__":
 	sys.stdout.write("Rank transform euclidean distance, and then apply exponential transform\n")
 	t_matrix = spatial_genes.rank_transform_matrix(euc, reverse=False, rbp_p=rbp_p, matrix_type=matrix_type)
 	sys.stdout.write("Compute silhouette metric per gene\n")
-
+	source_path = os.path.dirname(silhouetteRank.__file__)
 	if not os.path.isfile("%s/do_gpd.R" % outdir):
-		copyfile("do_gpd.R", "%s/do_gpd.R" % outdir)
+		copyfile("%s/do_gpd.R" % source_path, "%s/do_gpd.R" % outdir)
+	if not os.path.isfile("%s/do_kmeans.R" % outdir):
+		copyfile("%s/do_kmeans.R" % source_path, "%s/do_kmeans.R" % outdir)
+
 
 	res = spatial_genes.random_pattern(matrix=t_matrix, matrix_type=matrix_type, num_cell = ncell, sizes=size_to_do, trials_per_gene=5000, run_gpd=True, outdir=outdir)
