@@ -5,13 +5,13 @@ import numpy as np
 import subprocess
 import math
 import scipy
-import spatial_genes
+import silhouetteRank
+import silhouetteRank.spatial_genes as spatial_genes
 from shutil import copyfile
 from operator import itemgetter
 from scipy.spatial.distance import squareform, pdist
 from scipy.stats import percentileofscore
 from sklearn.metrics import roc_auc_score
-from sklearn.cluster import KMeans
 import argparse
 
 def read_matrix(f_expr="expression.txt", f_Xcen = "Xcen.good"):
@@ -19,6 +19,7 @@ def read_matrix(f_expr="expression.txt", f_Xcen = "Xcen.good"):
 	#f_Xcen = "Xcen.good"
 
 	sys.stderr.write("Reading gene expression...\n")
+	sys.stderr.flush()
 	f = open(f_expr)
 	h = f.readline().rstrip("\n").split("\t")[1:]
 	num_cell = len(h)
@@ -43,6 +44,7 @@ def read_matrix(f_expr="expression.txt", f_Xcen = "Xcen.good"):
 	f.close()
 
 	sys.stderr.write("Reading cell coordinates...\n")
+	sys.stderr.flush()
 	f = open(f_Xcen)
 	Xcen = np.empty((num_cell, 2), dtype="float32")
 	f.readline()
@@ -99,12 +101,32 @@ def read_frequency(expr=None, genes=None, Xcen=None, frequency_file=None, read_f
 				freq.append(int(ll[1]))
 		f.close()
 
-	kmeans = KMeans(n_clusters = num_query_sizes, random_state=0, n_init=1000, verbose=0).fit(np.array([freq]).T)
+	if len(sizes)<=num_query_sizes:
+		sys.stderr.write("Setting num_query_sizes to be %d instead of %d...\n" % (len(sizes), num_query_sizes))
+		sys.stderr.flush()
+		num_query_sizes = len(sizes) 
+	cmd = "cd '%s' && Rscript do_kmeans.R gene.freq.good.txt 1 %d 1000 km_centroid.txt km_label.txt" % (outdir, num_query_sizes)
+	print(cmd)
+	os.system(cmd)
+
+	f = open("%s/km_centroid.txt" % outdir)
 	cc = []
-	for c in kmeans.cluster_centers_:
-		cc.append(int(c[0]))
+	for l in f:
+		l = l.rstrip("\n")
+		ll = l.split(" ")
+		cc.append(int(float(ll[1])))
+	f.close()
+
+	labels = []
+	f = open("%s/km_label.txt" % outdir)
+	for l in f:
+		l = l.rstrip("\n")
+		ll = l.split(" ")
+		labels.append(int(ll[1]) - 1)
+	f.close()
+
 	by_cluster = {}
-	for ind,c in enumerate(kmeans.labels_):
+	for ind,c in enumerate(labels):
 		by_cluster.setdefault(cc[c], [])
 		by_cluster[cc[c]].append(freq[ind])
 
@@ -127,6 +149,7 @@ def do_one(args):
 	result = subprocess.call("Rscript --version", shell=True)
 	if result==127:
 		sys.stderr.write("Rscript is not found")
+		sys.stderr.flush()
 		sys.exit(1)	
 
 	if not os.path.isdir(args.output):
@@ -146,8 +169,10 @@ def do_one(args):
 	if t_overwrite:
 		expr, genes, Xcen = read_matrix(f_expr=args.expr, f_Xcen=args.centroid)
 		sys.stderr.write("Calculate all pairwise Euclidean distance between cells using their physical coordinates\n")
+		sys.stderr.flush()
 		euc = squareform(pdist(Xcen, metric="euclidean"))
 		sys.stderr.write("Rank transform euclidean distance, and then apply exponential transform\n")
+		sys.stderr.flush()
 		for rbp_p in args.rbp_ps:
 			t_matrix = spatial_genes.rank_transform_matrix(euc, reverse=False, rbp_p=rbp_p, matrix_type=matrix_type)
 			np.save("%s/t_matrix_%s_%.2f.npy" % (args.output, args.matrix_type, rbp_p), t_matrix)
@@ -156,6 +181,7 @@ def do_one(args):
 		np.save("%s/genes.npy" % args.output, genes)
 	else:
 		sys.stderr.write("Using existing input binaries...\n")
+		sys.stderr.flush()
 		expr = np.load("%s/expr.npy" % args.output)
 		Xcen = np.load("%s/Xcen.npy" % args.output)
 		genes = np.load("%s/genes.npy" % args.output)
@@ -169,10 +195,15 @@ def do_one(args):
 				outdir = "%s/result_5000_%.2f_%.3f" % (args.output, rbp_p, examine_top)
 			if not os.path.isdir(outdir):
 				os.mkdir(outdir)
+			source_path = os.path.dirname(silhouetteRank.__file__)
+			if not os.path.isfile("%s/do_kmeans.R" % outdir):
+				copyfile("%s/do_kmeans.R" % source_path, "%s/do_kmeans.R" % outdir)
+			if not os.path.isfile("%s/do_gpd.R" % outdir):
+				copyfile("%s/do_gpd.R" % source_path, "%s/do_gpd.R" % outdir)
+			if not os.path.isfile("%s/qval.R" % outdir):
+				copyfile("%s/qval.R" % source_path, "%s/qval.R" % outdir)
 			sizes = read_frequency(expr=expr, genes=genes, Xcen=Xcen, frequency_file=None, \
 			read_from_file=False, outdir=outdir, examine_top=examine_top, num_query_sizes=args.query_sizes)
-			if not os.path.isfile("%s/do_gpd.R" % outdir):
-				copyfile("do_gpd.R", "%s/do_gpd.R" % outdir)
 	
 if __name__=="__main__":
 	parser = argparse.ArgumentParser(description="evaluate.exact.2b.py: calculate silhouette score for randomly distributed spatial patterns", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
