@@ -5,72 +5,18 @@ import numpy as np
 import subprocess
 import math
 import scipy
-import spatial_genes
+import silhouetteRank.spatial_genes as spatial_genes
 from shutil import copyfile
 from operator import itemgetter
 from scipy.spatial.distance import squareform, pdist
 from scipy.stats import percentileofscore
 from sklearn.metrics import roc_auc_score
-from sklearn.cluster import KMeans
 import argparse
-import prep
-import evaluate_exact_one_2b
-import use_previous_cluster
-import combine
-
-def do_one(args):
-	matrix_type = args.matrix_type
-	rbp_ps = args.rbp_ps
-	examine_tops = args.examine_tops
-	result = subprocess.call("Rscript --version", shell=True)
-	if result==127:
-		sys.stderr.write("Rscript is not found")
-		sys.exit(1)	
-
-	if not os.path.isdir(args.output):
-		os.mkdir(args.output)
-	
-	t_overwrite = args.overwrite_input_bin
-	#if there is no binary input file, create one
-	check_required = ["expr.npy", "Xcen.npy", "genes.npy"]
-	for rbp_p in args.rbp_ps:
-		check_required.append("t_matrix_%s_%.2f.npy" % (args.matrix_type, rbp_p))
-	for cr in check_required:
-		if not os.path.isfile("%s/%s" % (args.output, cr)):
-			t_overwrite = True
-			break
-
-	expr, Xcen, genes, t_matrix = None, None, None, None
-	if t_overwrite:
-		expr, genes, Xcen = read_matrix(f_expr=args.expr, f_Xcen=args.centroid)
-		sys.stderr.write("Calculate all pairwise Euclidean distance between cells using their physical coordinates\n")
-		euc = squareform(pdist(Xcen, metric="euclidean"))
-		sys.stderr.write("Rank transform euclidean distance, and then apply exponential transform\n")
-		for rbp_p in args.rbp_ps:
-			t_matrix = spatial_genes.rank_transform_matrix(euc, reverse=False, rbp_p=rbp_p, matrix_type=matrix_type)
-			np.save("%s/t_matrix_%s_%.2f.npy" % (args.output, args.matrix_type, rbp_p), t_matrix)
-		np.save("%s/expr.npy" % args.output, expr)
-		np.save("%s/Xcen.npy" % args.output, Xcen)
-		np.save("%s/genes.npy" % args.output, genes)
-	else:
-		sys.stderr.write("Using existing input binaries...\n")
-		expr = np.load("%s/expr.npy" % args.output)
-		Xcen = np.load("%s/Xcen.npy" % args.output)
-		genes = np.load("%s/genes.npy" % args.output)
-		#t_matrix = np.load("%s/t_matrix_%s_%.2f.npy" % (args.output, args.matrix_type, args.rbp_p))	
-	
-	ncell = Xcen.shape[0] 
-	for rbp_p in args.rbp_ps:
-		for examine_top in args.examine_tops:
-			outdir = "%s/result_sim_5000_%.2f_%.3f" % (args.output, rbp_p, examine_top)
-			if matrix_type=="dissim":
-				outdir = "%s/result_5000_%.2f_%.3f" % (args.output, rbp_p, examine_top)
-			if not os.path.isdir(outdir):
-				os.mkdir(outdir)
-			sizes = read_frequency(expr=expr, genes=genes, Xcen=Xcen, frequency_file=None, \
-			read_from_file=False, outdir=outdir, examine_top=examine_top, num_query_sizes=args.query_sizes)
-			if not os.path.isfile("%s/do_gpd.R" % outdir):
-				copyfile("do_gpd.R", "%s/do_gpd.R" % outdir)
+import silhouetteRank
+import silhouetteRank.prep as prep
+import silhouetteRank.evaluate_exact_one_2b as evaluate_exact_one_2b
+import silhouetteRank.use_previous_cluster as use_previous_cluster
+import silhouetteRank.combine as combine
 
 def main():
 	parser = argparse.ArgumentParser(description="silhouette_rank_one.py: calculate silhouette score for randomly distributed spatial patterns", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
@@ -87,10 +33,31 @@ def main():
 	parser.add_argument("-q", "--query-sizes", dest="query_sizes", type=int, default=10, help="query sizes (advanced user setting)")
 	args = parser.parse_args()
 
+	if not os.path.isdir(args.output):
+		os.mkdir(args.output)
+	if not os.path.isdir(args.logdir):
+		os.mkdir(args.logdir)
+
+	args1 = argparse.Namespace(expr=args.expr, centroid=args.centroid, rbp_ps=args.rbp_ps, examine_tops=args.examine_tops, matrix_type=args.matrix_type, output=args.output, query_sizes=args.query_sizes, overwrite_input_bin=args.overwrite_input_bin)
+	prep.do_one(args1)
+
+
 	fw = open("%s/args" % args.output, "w")
 	for rbp_p in args.rbp_ps:
 		for examine_top in args.examine_tops:
-			for i in range(args.query_sizes):
+			freq_file = "%s/result_5000_%.2f_%.3f/gene.freq.good.txt" % (args.output, rbp_p, examine_top)
+			if args.matrix_type=="sim":
+				freq_file = "%s/result_sim_5000_%.2f_%.3f/gene.freq.good.txt" % (args.output, rbp_p, examine_top)
+			uniq_freq = 0
+			f = open(freq_file)
+			for l in f:
+				l = l.rstrip("\n")
+				uniq_freq+=1
+			f.close()
+			num_query_sizes = args.query_sizes
+			if uniq_freq<=num_query_sizes:
+				num_query_sizes = uniq_freq
+			for i in range(num_query_sizes):
 				fw.write("%.2f\n" % rbp_p)
 				fw.write("%.3f\n" % examine_top)
 				fw.write("%d\n" % i)
@@ -102,19 +69,21 @@ def main():
 			fw.write("%.2f\n" % rbp_p)
 			fw.write("%.3f\n" % examine_top)
 	fw.close()
-	
 
-	args1 = argparse.Namespace(expr=args.expr, centroid=args.centroid, rbp_ps=args.rbp_ps, examine_tops=args.examine_tops, matrix_type=args.matrix_type, output=args.output, query_sizes=args.query_sizes, overwrite_input_bin=args.overwrite_input_bin)
-	prep.do_one(args1)
+	bin_path = os.path.dirname(silhouetteRank.__file__)
+	for i in range(4):
+		bin_path = os.path.dirname(bin_path)
+	bin_path = os.path.join(bin_path, "bin")	
 
-	if not os.path.isdir(args.logdir):
-		os.mkdir(args.logdir)
+	sys.stderr.write("Start calculating silhouette rank...\n")
+	sys.stderr.flush()
+	cmd = "cat '%s'/args.basic | '%s'/parallel --jobs %d --max-args=2 \\''%s'\\'''/silhouette_rank_main -x \\''%s'\\''' -c \\''%s'\\''' -r {1} -e {2} -m %s -o \\''%s'\\''' \"2>\" \\''%s'\\'''/real_{1}_{2}.out" % (args.output, args.parallel_path, args.num_core, bin_path, args.expr, args.centroid, args.matrix_type, args.output, args.logdir)
+	os.system(cmd)
 
-	print("Start calculating silhouette rank...")
-	os.system("cat args.basic|%s/parallel --jobs %d --max-args=2 python3 -u evaluate_2b.py -x %s -c %s -r {1} -e {2} -m %s -o . \"2>\" %s/real_{1}_{2}.out" % (args.parallel_path, args.num_core, args.expr, args.centroid, args.matrix_type, args.logdir))
-
-	print("Start random...")
-	os.system("cat args|%s/parallel --jobs %d --max-args=3 python3 -u evaluate_exact_one_2b.py -r {1} -e {2} -m %s -o . -q {3} \"2>\" %s/{1}_{2}_{3}.out" % (args.parallel_path, args.num_core, args.matrix_type, args.logdir))
+	sys.stderr.write("Start random...\n")
+	sys.stderr.flush()
+	cmd="cat '%s'/args | '%s'/parallel --jobs %d --max-args=3 \\''%s'\\'''/silhouette_rank_random -r {1} -e {2} -m %s -o \\''%s'\\''' -q {3} \"2>\" \\''%s'\\'''/{1}_{2}_{3}.out" % (args.output, args.parallel_path, args.num_core, bin_path, args.matrix_type, args.output, args.logdir)
+	os.system(cmd)
 
 	for rbp_p in args.rbp_ps:
 		for examine_top in args.examine_tops:
@@ -131,14 +100,11 @@ def main():
 			args1 = argparse.Namespace(expr=args.expr, centroid=args.centroid, examine_top=examine_top, input=score_file, input_random=random_dir, output=output_score_file, outdir=args.output, query_sizes=args.query_sizes, overwrite_input_bin=args.overwrite_input_bin)
 			use_previous_cluster.do_one(args1)
 
-	combined_file = "silhouette.overall.pval.txt"
+	combined_file = "%s/silhouette.overall.pval.txt" % args.output
 	if args.matrix_type=="sim":
-		combined_file = "silhouette.sim.overall.pval.txt"
+		combined_file = "%s/silhouette.sim.overall.pval.txt" % args.output
 	args1 = argparse.Namespace(rbp_ps=args.rbp_ps, examine_tops=args.examine_tops, matrix_type=args.matrix_type, input=args.output, output=combined_file)
 	combine.do_one(args1)
-	#for i in range(args.query_sizes):
-	#	evaluate_exact_one_2b.do_one(args)
-	#do_one(args)
 	
 if __name__=="__main__":
 	main()
