@@ -35,6 +35,7 @@ def rank_transform_matrix(mat, rbp_p = 0.99, reverse=True, matrix_type="dissim")
 	dim2 = mat.shape[1]
 	rank_forward = np.empty([dim1, dim2])
 	sys.stderr.write("Start ranking forward...\n")
+	sys.stderr.flush()
 	for c1 in range(dim1):
 		rd = scipy.stats.rankdata(mat[c1,:])
 		if reverse==True:
@@ -42,9 +43,12 @@ def rank_transform_matrix(mat, rbp_p = 0.99, reverse=True, matrix_type="dissim")
 		rank_forward[c1, :] = rd
 		if c1%1000==0:
 			sys.stderr.write("Done %d\n" % c1)
+			sys.stderr.flush()
 	sys.stderr.write("Finished ranking forward...\n")
+	sys.stderr.flush()
 	rank_backward = np.empty([dim1, dim2])
 	sys.stderr.write("Start ranking backward...\n")
+	sys.stderr.flush()
 	for c1 in range(dim2):
 		rd = scipy.stats.rankdata(mat[:,c1])
 		if reverse==True:
@@ -52,23 +56,32 @@ def rank_transform_matrix(mat, rbp_p = 0.99, reverse=True, matrix_type="dissim")
 		rank_backward[:, c1] = rd
 		if c1%1000==0:
 			sys.stderr.write("Done %d\n" % c1)
+			sys.stderr.flush()
 	mutual_rank_rbp = np.empty([dim1, dim2])
 	mutual_rank = np.empty([dim1, dim2])
 	sys.stderr.write("Calculate mutual rank...\n")
+	sys.stderr.flush()
 	ma = np.sqrt(np.multiply(rank_forward, rank_backward))
 	if matrix_type=="dissim":
 		sys.stderr.write("Calculate exponential transform...\n")
+		sys.stderr.flush()
 		dissimilarity = np.subtract(1, np.power(rbp_p, np.subtract(ma, 1)))
 		sys.stderr.write("Finished exponential transform...\n")
+		sys.stderr.flush()
 		sys.stderr.write("Finished dissimilarity...\n")
+		sys.stderr.flush()
 		return dissimilarity
 	if matrix_type=="sim":
 		sys.stderr.write("Calculate exponential transform...\n")
+		sys.stderr.flush()
 		similarity = np.power(rbp_p, np.subtract(ma, 1))
 		sys.stderr.write("Finished exponential transform...\n")
+		sys.stderr.flush()
 		sys.stderr.write("Finished similarity...\n")
+		sys.stderr.flush()
 		return similarity	
 	sys.stderr.write("Error: matrix type is not recognized\n")
+	sys.stderr.flush()
 	return None
 
 #matrix type is either dissim or sim
@@ -87,6 +100,7 @@ def random_pattern(matrix=None, matrix_type="dissim", num_cell=None, sizes=None,
 		rand_clust[aSize:] = 2
 		rand_sil.setdefault(aSize, [])
 		sys.stderr.write("Getting through size %d, %d of %d\n" % (aSize, ind, len(sizes)))
+		sys.stderr.flush()
 		repeats.setdefault(aSize, 0)
 		repeats[aSize]+=1
 		trials = []
@@ -98,12 +112,14 @@ def random_pattern(matrix=None, matrix_type="dissim", num_cell=None, sizes=None,
 			for i in range(trials_per_gene):
 				if i%100==0:
 					sys.stderr.write("%d\n" % i)
+					sys.stderr.flush()
 				r_avg_clust1_sil = get_distance_per_FD_2(matrix, ncell, trials[i], outcome=[1,2])
 				rand_sil[aSize].append(r_avg_clust1_sil)
 		elif matrix_type=="sim":
 			for i in range(trials_per_gene):
 				if i%100==0:
 					sys.stderr.write("%d\n" % i)
+					sys.stderr.flush()
 				r_avg_clust1_sil = get_distance_per_FD_3(matrix, ncell, trials[i], outcome=[1,2])
 				rand_sil[aSize].append(r_avg_clust1_sil)
 		fw = open("%s/%d" % (outdir, aSize), "w")
@@ -112,7 +128,7 @@ def random_pattern(matrix=None, matrix_type="dissim", num_cell=None, sizes=None,
 		fw.close()
 		re_run = False
 		if run_gpd:
-			os.system("cd %s && Rscript do_gpd.R %d" % (outdir, aSize))
+			os.system("cd '%s' && Rscript do_gpd.R %d" % (outdir, aSize))
 			f = open("%s/par.%d" % (outdir, aSize))
 			scale = float(f.readline().rstrip("\n").split("\t")[1])
 			shape = float(f.readline().rstrip("\n").split("\t")[1])
@@ -122,6 +138,7 @@ def random_pattern(matrix=None, matrix_type="dissim", num_cell=None, sizes=None,
 		if re_run:
 			if repeats[aSize]>=5:
 				sys.stderr.write("Repeats reached, going to next one...\n")
+				sys.stderr.flush()
 			else:
 				rand_sil[aSize] = []
 				ind -= 1
@@ -131,10 +148,12 @@ def random_pattern(matrix=None, matrix_type="dissim", num_cell=None, sizes=None,
 def calc_silhouette_per_gene_approx(genes=None, expr=None, matrix=None, matrix_type="dissim", examine_top=0.1, seed=-1):
 	if genes is None or expr is None or matrix is None:
 		sys.stderr.write("Need genes, expr, similarity/dissimilarity matrix\n")
+		sys.stderr.flush()
 		return ;
 	if seed!=-1 and seed>=0:
 		np.random.seed(seed)
 	sys.stderr.write("Started 2 " + "\n")
+	sys.stderr.flush()
 	sil = []
 	ncell = expr.shape[1]
 	ex = int((1.0-examine_top)*100.0)
@@ -160,6 +179,7 @@ def calc_silhouette_per_gene_approx(genes=None, expr=None, matrix=None, matrix_t
 			clust[lt] = 2
 		if ig%100==0:
 			sys.stderr.write("%s %d / %d\n" % (g, ig, len(genes)))
+			sys.stderr.flush()
 		if matrix_type=="dissim":
 			avg_clust1_sil = get_distance_per_FD_2(matrix, ncell, clust, outcome=[1,2])
 		elif matrix_type=="sim":
@@ -177,10 +197,12 @@ def calc_silhouette_per_gene_approx(genes=None, expr=None, matrix=None, matrix_t
 def calc_silhouette_per_gene(genes=None, expr=None, matrix=None, matrix_type="dissim", examine_top=0.1, seed=-1):
 	if genes is None or expr is None or matrix is None:
 		sys.stderr.write("Need genes, expr, dissim/sim matrix\n")
+		sys.stderr.flush()
 		return ;
 	if seed!=-1 and seed>=0:
 		np.random.seed(seed)
 	sys.stderr.write("Started 2 " + "\n")
+	sys.stderr.flush()
 	sil = []
 	ncell = expr.shape[1]
 	ex = int((1.0-examine_top)*100.0)
@@ -196,6 +218,7 @@ def calc_silhouette_per_gene(genes=None, expr=None, matrix=None, matrix_type="di
 		clust[lt] = 2
 		if ig%100==0:
 			sys.stderr.write("%s %d / %d\n" % (g, ig, len(genes)))
+			sys.stderr.flush()
 		if matrix_type=="dissim":
 			avg_clust1_sil = get_distance_per_FD_2(matrix, ncell, clust, outcome=[1,2])
 		elif matrix_type=="sim":
@@ -222,10 +245,13 @@ def python_spatial_genes(spatial_locations, expression_matrix, metric = "euclide
     
 	ncell = Xcen.shape[0] 
 	sys.stderr.write("Calculate all pairwise Euclidean distance between cells using their physical coordinates\n")
+	sys.stderr.flush()
 	euc = squareform(pdist(Xcen, metric=metric))
 	sys.stderr.write("Rank transform euclidean distance, and then apply exponential transform\n")
+	sys.stderr.flush()
 	dissim = rank_transform_matrix(euc, reverse=False, rbp_p=rbp_p)
 	sys.stderr.write("Compute silhouette metric per gene\n")
+	sys.stderr.flush()
 	res = calc_silhouette_per_gene(genes=genes, expr=expr, dissim=dissim, examine_top=examine_top)
     
 	return res
