@@ -82,7 +82,7 @@ def read(f_expr="expression.txt", f_Xcen="Xcen.good"):
 		Xcen[ic,:] = [float(ll[1]), float(ll[2])]
 		ic+=1
 	f.close()
-	return expr, Xcen, genes, pattern_size
+	return expr, Xcen, genes
 
 def do_one(args):
 	result = subprocess.call("Rscript --version", shell=True)
@@ -92,9 +92,23 @@ def do_one(args):
 		sys.exit(1)	
 
 	outdir=args.input_random
+
+	uniq_freq = 0
+	f = open("%s/gene.freq.good.txt" % outdir)
+	for l in f:
+		l = l.rstrip("\n")
+		uniq_freq+=1
+	f.close()
+	
 	cc = []
 	param = {}
-	alist = ["%d" % x for x in list(range(args.query_sizes))]
+	num_query_sizes = args.query_sizes
+	sys.stderr.write("Setting query size %d instead of %d\n" % (uniq_freq, num_query_sizes))
+	sys.stderr.flush()
+	if uniq_freq<=num_query_sizes:
+		num_query_sizes = uniq_freq
+
+	alist = ["%d" % x for x in list(range(num_query_sizes))]
 	mapto = {}	
 	full_list = {}
 	for i in alist:
@@ -184,7 +198,7 @@ def do_one(args):
 		for i in by_size[targ]:
 			fw.write(str(i) + "\n")
 		fw.close()
-		os.system("cd %s && Rscript qval.R /tmp/1 /tmp/1.qval" % args.outdir)
+		os.system("cd '%s' && Rscript qval.R /tmp/1 /tmp/1.qval" % outdir)
 		s_scores = []
 		f = open("/tmp/1.qval")
 		for l in f:
